@@ -2,8 +2,8 @@
 LogPy is a Python library for logic and relational programming.
 """
 
-from core import run, eq, var, conde, membero
+from core import run, eq, var, isvar, conde, membero
 from goals import seteq
 from facts import Relation, fact, facts
 
-__version__ = '0.1.8'
+__version__ = '0.1.9'
